{% extends "base.html" %}

{% block extrahead %}
  <meta property="og:title" content="{{ page.title }} - Thanakan" />
  <meta property="og:description" content="เครื่องมือจัดการธนาคารไทยครบวงจร - Thai Bank Utilities" />
  <meta property="og:image" content="https://raw.githubusercontent.com/ninyawee/thanakan/main/docs/assets/og-image.jpg" />
  <meta property="og:url" content="{{ page.canonical_url }}" />
  <meta property="og:type" content="website" />
  <meta name="twitter:card" content="summary_large_image" />
  <meta name="twitter:title" content="{{ page.title }} - Thanakan" />
  <meta name="twitter:description" content="เครื่องมือจัดการธนาคารไทยครบวงจร - Thai Bank Utilities" />
  <meta name="twitter:image" content="https://raw.githubusercontent.com/ninyawee/thanakan/main/docs/assets/og-image.jpg" />
{% endblock %}

{% block content %}
  {{ super() }}

  <h2 id="__comments">{{ lang.t("meta.comments") }}</h2>

  <script
    src="https://giscus.app/client.js"
    data-repo="ninyawee/thanakan"
    data-repo-id="R_kgDOQlmEyQ"
    data-category="General"
    data-category-id="DIC_kwDOQlmEyc4Czlsg"
    data-mapping="pathname"
    data-strict="0"
    data-reactions-enabled="1"
    data-emit-metadata="0"
    data-input-position="bottom"
    data-theme="preferred_color_scheme"
    data-lang="th"
    data-loading="lazy"
    crossorigin="anonymous"
    async
  ></script>
{% endblock %}
